<?php

/**
 * @property-read string $shop_id
 * @property-read string $secret_key
 *
 *
 */
class sprypayPayment extends waPayment implements waIPayment
{
    public function allowedCurrency()
    {
        $currency = array('RUB', 'USD', 'EUR', 'UAH');
        return $currency;
    }

    public function payment($payment_form_data, $order_data, $auto_submit = false)
    {

        $order = waOrder::factory($order_data);

        $form_fields = array();
        $form_fields['spShopId'] = $this->shop_id;
		$form_fields['spAmount'] = number_format($order->total, 2, '.', '');
        $form_fields['spShopPaymentId'] = $order->id;
        $form_fields['spPurpose'] = htmlentities($order->description, ENT_QUOTES, 'utf-8');
        $form_fields['spUserEmail'] = $this->email;
        if($order->currency == "RUB")  $form_fields['spCurrency']="rur";
        else $form_fields['spCurrency'] = strtolower($order->currency);
        $form_fields['lang'] = ($this->locale) ? $this->locale : 'en';
        $form_fields['src'] = 1;
        $form_fields['spSuccessUrl'] = $this->getAdapter()->getBackUrl(waAppPayment::URL_SUCCESS, array('order_id' => $order->id));
        $form_fields['spSuccessMethod'] = 1;
        $form_fields['spFailUrl'] = $this->getAdapter()->getBackUrl(waAppPayment::URL_FAIL, array('order_id' => $order->id));
        $form_fields['spFailMethod'] = 1;
        $form_fields['spIpnUrl'] = $this->getRelayUrl();
        $form_fields['spIpnMethod'] = 1;
        if(strpos($form_fields['spIpnUrl'], "?"))
            $form_fields['spIpnUrl'] .= '&app_id=' . $this->app_i . '&merchant_id=' . $this->merchant_id;
        else
            $form_fields['spIpnUrl'] .= '?app_id=' . $this->app_id . '&merchant_id=' . $this->merchant_id;
        $auto_submit=true;
        $view = wa()->getView();

        $view->assign('form_fields', $form_fields);
        $view->assign('form_url', 'https://sprypay.ru/sppi/');
        $view->assign('auto_submit', $auto_submit);

        return $view->fetch($this->path.'/templates/payment.html');
    }

    protected function callbackInit($request)
    {
        if (!empty($request['spShopPaymentId']) && intval($request['spShopPaymentId'])) {
            $this->app_id = $request['app_id'];
            $this->merchant_id = $request['merchant_id'];
            $this->order_id = intval($request['spShopPaymentId']);
        }
        else die("error no ipn data");

        return parent::callbackInit($request);
    }

    public function callbackHandler($request)
    {

        $transaction_data = $this->formalizeData($request);
        $transaction_result = ifempty($request['status'], 'result');

        $url = null;
        $app_payment_method = null;

        switch ($transaction_result) {
            case 'result':
                $this->verifySign($request);
                $app_payment_method = self::CALLBACK_PAYMENT;
                $transaction_data['state'] = self::STATE_CAPTURED;
                break;
            case '1':
                $url = $this->getAdapter()->getBackUrl(waAppPayment::URL_SUCCESS, $transaction_data);
                break;
            case '0':
                if ($this->order_id && $this->app_id) {
                    $app_payment_method = self::CALLBACK_CANCEL;
                    $transaction_data['state'] = self::STATE_CANCELED;
                }
            default:
                $url = $this->getAdapter()->getBackUrl(waAppPayment::URL_FAIL, $transaction_data);
                break;
        }

        $transaction_data = $this->saveTransaction($transaction_data, $request);
        if ($app_payment_method) {
            $result = $this->execAppCallback($app_payment_method, $transaction_data);
            self::addTransactionData($transaction_data['id'], $result);

        }
        if ($transaction_result == 'result') {
            echo 'ok';
            return array(
                'template' => false,
            );
        } else {
            if ($url) {
                return array(
                    'redirect' => $url,
                );
            } else {
                return array(
                    'template' => $this->path.'/templates/callback.html',
                    'back_url' => $url,
                    'message'  => $message,
                );
            }
        }
    }

    protected function formalizeData($transaction_raw_data)
    {

        $transaction_data = parent::formalizeData($transaction_raw_data);
        $transaction_data['native_id'] = $transaction_raw_data['spShopPaymentId'];
        $transaction_data['order_id'] = $transaction_raw_data['spShopPaymentId'];
        $transaction_data['amount'] = ifempty($transaction_raw_data['spAmount'], '');
        if($transaction_raw_data['spCurrency'] == 'rur') $transaction_data['currency_id'] = "RUB";
        else $transaction_data['currency_id'] = strtoupper($transaction_raw_data['spCurrency']);

        return $transaction_data;
    }

    private function verifySign($request)
    {
		if ($this->secret_key) {
        	 $hash_string = $request['spPaymentId'].$request['spShopId'].$request['spShopPaymentId'].
        				$request['spBalanceAmount'].$request['spAmount'].$request['spCurrency'].
        				$request['spCustomerEmail'].$request['spPurpose'].$request['spPaymentSystemId'].
        				$request['spPaymentSystemAmount'].$request['spPaymentSystemPaymentId'].$request['spEnrollDateTime'].
        				$this->secret_key;
	        $hash = strtolower(md5($hash_string));
	        $sign = $request['spHashString'];
	        if (empty($sign) || ($sign != $hash)) {

	            die("error hash: ".$hash." sign: ".$sign);
	        }
        }
        else die("error secret_key");

    }
}
